from csirtgsdk.client.http import HTTP as Client


class Feed(object):
    """
    Represents a Feed Object
    """
    def __init__(self, client=Client()):
        self.client = client

    def get_lines(self, data):
        from prettytable import PrettyTable
        cols = ['name', 'description', 'license', 'updated_at']
        t = PrettyTable(cols)
        if isinstance(data, dict):
            data = [data]

        for f in data:
            r = []

            for c in cols:
                y = f.get(c)
                if c == 'license':
                    y = y['name']
                y = str(y)
                y = (y[:30] + '..') if len(y) > 30 else y
                r.append(y)
            t.add_row(r)

        yield str(t)

    def new(self, user, name, description=None):
        """
        Creates a new Feed object

        :param user: feed username
        :param name: feed name
        :param description: feed description
        :return: dict
        """
        uri = self.client.remote + '/users/{0}/feeds'.format(user)

        data = {
            'feed': {
                'name': name,
                'description': description
            }
        }

        resp = self.client.post(uri, data)
        return resp

    def delete(self, user, name):
        """
        Removes a feed

        :param user: feed username
        :param name: feed name
        :return: true/false
        """

        uri = self.client.remote + '/users/{}/feeds/{}'.format(user, name)

        resp = self.client.session.delete(uri)
        return resp.status_code

    remove = delete

    def index(self, user):
        """
        Returns a list of Feeds from the API

        :param user: feed username
        :return: list

        Example:
            ret = feed.index('csirtgadgets')
        """
        uri = self.client.remote + '/users/{0}/feeds'.format(user)
        return self.client.get(uri)

    def show(self, user, name, limit=None, lasttime=None):
        """
        Returns a specific Feed from the API

        :param user: feed username
        :param name: feed name
        :param limit: limit the results
        :param lasttime: only show >= lasttime
        :return: dict

        Example:
            ret = feed.show('csirtgadgets', 'port-scanners', limit=5)
        """
        uri = self.client.remote + '/users/{0}/feeds/{1}'.format(user, name)
        return self.client.get(uri, params={'limit': limit, 'lasttime': lasttime})





from csirtgsdk.constants import TOKEN, REMOTE

from csirtgsdk.feed import Feed
from csirtgsdk.indicator import Indicator
from csirtgsdk.search import Search
from csirtgsdk.client.http import HTTP as Client


def feed(f, limit=25):
    """
    Pull a feed
    :param f: feed name (eg: csirtgadgetes/correlated)
    :param limit: return value limit (default 25)
    :return: Feed dict
    """
    if '/' not in f:
        raise ValueError('feed name must be formatted like: '
                         'csirtgadgets/scanners')

    user, f = f.split('/')

    return Feed().show(user, f, limit=limit)


def search(i, limit=25):
    """
    Search for an indicator
    :param i: indicator (eg: example.com, 192.168.1.1..)
    :param limit: result set limit
    :return: list of dicts
    """

    # Search for an indicator
    return Search().search(i, limit=limit)


def indicator_create(f, i):
    """
    Create an indicator in a feed
    :param f: feed name (eg: wes/test)
    :param i: indicator dict (eg: {'indicator': 'example.com', 'tags': ['ssh'],
    'description': 'this is a test'})
    :return: dict of indicator
    """
    if '/' not in f:
        raise ValueError('feed name must be formatted like: '
                         'csirtgadgets/scanners')

    if not i:
        raise ValueError('missing indicator dict')

    u, f = f.split('/')

    i['user'] = u
    i['feed'] = f

    ret = Indicator(i).submit()

    return ret



import pytest

from csirtgsdk.feed import Feed
from csirtgsdk.client.http import HTTP as Client

@pytest.fixture
def client():
    return Client()


def test_feed(client):
    f = Feed(client)

    assert f.client


import os.path
import sys

from ._version import get_versions
VERSION = get_versions()['version']
del get_versions

API_VERSION = os.getenv('CSIRTG_API_VERSION', '1')

REMOTE = 'https://csirtg.io/api'
REMOTE = os.getenv('CSIRTG_REMOTE', REMOTE)

TOKEN = os.getenv('CSIRTG_TOKEN', None)

LIMIT = 25
LIMIT = os.getenv('CSIRTG_LIMIT', LIMIT)

TIMEOUT = 30
TIMEOUT = os.getenv('CSIRTG_TIMEOUT', TIMEOUT)

COLUMNS = "user,feed,indicator,firsttime,lasttime,count,comments,protocol," \
          "portlist,tags,description,cc,asn,asn_desc"
COLUMNS = os.getenv('CSIRTG_COLUMNS', COLUMNS)
COLUMNS = COLUMNS.split(',')

MAX_FIELD_SIZE = 30
MAX_FIELD_SIZE = os.getenv('CSIRTG_MAX_FIELD_SIZE', MAX_FIELD_SIZE)

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'

PYVERSION = 2
if sys.version_info > (3,):
    PYVERSION = 3



import pytest
import os
from time import sleep
from csirtgsdk.feed import Feed
from csirtgsdk.client.http import HTTP as Client

CI_BUILD = os.environ.get('CI_BUILD', False)
TOKEN = os.environ.get('CSIRTG_TOKEN', None)
USER = os.environ.get('CSIRTG_USER', 'wes')
REMOTE = os.environ.get('CSIRTG_REMOTE', 'https://csirtg.io/api')

liveonly = pytest.mark.skipif(CI_BUILD is False, reason="CI_BUILD env var not set")

@pytest.fixture
def client():
    return Client(
        token=TOKEN,
        remote=REMOTE
    )

@liveonly
def test_create_feed(client):
    sleep(3)
    # make sure feed isnt left over from a previous run
    f = Feed(client).remove(USER, 'ci_build_test')

    f = Feed(client).new(USER, 'ci_build_test', description='test build feed')

    assert f['created_at']

    f = Feed(client).remove(USER, 'ci_build_test')

    assert f == 200


@liveonly
def test_feed(client):
    f = Feed(client)

    assert f.client



import websocket
import logging
import os
import json
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import textwrap
from csirtgsdk.constants import LOG_FORMAT, TOKEN

try:
    import thread
except ImportError:
    import _thread as thread

REMOTE = 'wss://csirtg.io/firehose'

logger = logging.getLogger(__name__)

logger.setLevel(logging.ERROR)

if os.getenv('CSIRTGSDK_HTTP_TRACE', '0') == '1':
    logger.setLevel(logging.DEBUG)

logging.getLogger('websocket').setLevel(logging.WARN)

if os.getenv('CSIRTG_HTTP_WEBSOCKET_TRACE', '0') == '1':
    logging.getLogger('websocket').setLevel(logging.DEBUG)
    websocket.enableTrace(True)


class DefaultHandler(object):
    def __init__(self, remote=REMOTE, token=TOKEN):

        self.handle = websocket.WebSocketApp(
            remote,
            on_message=self.on_message,
            on_error=self.on_error,
            on_close=self.on_close,
            header={'Authorization:Token token=' + token}
        )

        self.error = False

    def on_admin_message(self, m):
        if not m.get('type'):
            return False

        if m['type'] in ['ping', 'welcome']:
            return True

        if m['type'] == 'confirm_subscription':
            msg = json.loads(m['identifier'])
            logger.info('subscribed to %s' % msg['channel'])

        return True

    def on_message(self, ws, message):
        logger.debug(message)

        m = json.loads(message)

        if self.on_admin_message(m):
            return

        print(m['message'])

    def on_error(self, ws, error):
        if len(str(error)):
            logger.error(error)
            self.error = True

    def on_close(self, ws):
        logger.info(' ### CLOSED ###')

    # https://github.com/tobiasfeistmantl/python-actioncable-zwei/blob/master/actioncable/subscription.py
    # https://github.com/NullVoxPopuli/action_cable_client#the-action-cable-protocol
    def on_open(self, ws):

        data = {
            'command': 'subscribe',
            'identifier': json.dumps({'channel': 'IndicatorsChannel'}),
        }
        ws.send(json.dumps(data))

    def run(self):
        self.handle.on_open = self.on_open
        self.handle.run_forever()


def main():
    class MyHandler(DefaultHandler):
        def on_close(self, ws):
            logger.info("Closed using MyHandler...")

    parser = ArgumentParser(
        description=textwrap.dedent('''\
            example usage:
              $ export CSIRTG_TOKEN=abcdefg1234...
                $ csirtg-firehose -v
            '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='csirtg-firehose'
    )

    parser.add_argument("-v", "--verbose", action="count", help="set verbosity level to INFO")
    parser.add_argument('-d', '--debug', action="store_true")
    parser.add_argument('-r', '--reconnect', action="store_true", help="auto reconnect if connection closes...", default=False)

    args = parser.parse_args()

    loglevel = logging.WARNING
    if args.verbose:
        loglevel = logging.INFO
    if args.debug:
        loglevel = logging.DEBUG

    logger.setLevel(loglevel)

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)

    h = MyHandler()

    while True:
        h.run()

        # if we closed and it wasn't an error
        if not h.error:
            break

        # if we didn't set the re-connect flag
        if not args.reconnect:
            break

        # we set the re-connect and it was an error, clear the error and re-connect.
        h.error = False

        logger.info('re-connecting..')


if __name__ == "__main__":
    main()



import pytest

from csirtgsdk.indicator import Indicator


def test_indicator():
    f = Indicator({
        'feed': 'testfeed',
        'user': 'testuser',
        'indicator': 'example.com',
        'tags': ['tag1', 'tag2']
    })

    assert f.client
    assert f.indicator.indicator == 'example.com'



import logging
import json
from pprint import pprint
from .constants import PYVERSION


class Sinkhole(object):
    """
    Sinkhole Object class
    """
    def __init__(self, client):
        """

        :param client: client.Client object
        :return: Search Object
        """
        self.logger = logging.getLogger(__name__)
        self.client = client

    def post(self, data):
        """
        POSTs a raw SMTP message to the Sinkhole API

        :param data: raw content to be submitted [STRING]
        :return: { list of predictions }
        """
        uri = '{}/sinkhole'.format(self.client.remote)
        self.logger.debug(uri)

        if PYVERSION == 2:
            try:
                data = data.decode('utf-8')
            except Exception:
                data = data.decode('latin-1')

        data = {
            'message': data
        }

        body = self.client.post(uri, data)
        return body




class CsirtgsdkException(Exception):
    def __init__(self, msg='Unknown'):
        self.msg = "{}".format(msg)

    def __str__(self):
        return self.msg


class AuthError(CsirtgsdkException):
    def __init__(self, msg='Unauthorized'):
        self.msg = msg


class TimeoutError(CsirtgsdkException):
    def __init__(self, msg='Timeout'):
        self.msg = msg


class InvalidSearch(CsirtgsdkException):
    def __init__(self, msg='Invalid Search'):
        self.msg = msg


class NotFound(CsirtgsdkException):
    def __init__(self, msg='Not Found'):
        self.msg = msg


class SubmissionFailed(CsirtgsdkException):
    def __init__(self, msg='Submission Failed'):
        self.msg = msg


class RateLimitExceeded(CsirtgsdkException):
    def __init__(self, msg='Rate Limit Threshold Exceeded'):
        self.msg = msg


class SystemBusy(CsirtgsdkException):
    def __init__(self, msg='System appears busy at the moment'):
        self.msg = msg



import logging
import base64
import hashlib
from csirtg_indicator import Indicator as I
import os.path
import json

from csirtgsdk.client.http import HTTP as Client


class Indicator(object):
    """
    Represents an Indicator object
    https://github.com/csirtgadgets/csirtgsdk/wiki/API#indicators
    """
    def __init__(self, kwargs):
        """
        :param client: csirtgsdk.client.Client object
        :param kwargs: dict of Indicator
        :return: Indicator object

        Example:
            Indicator({
                'indicator': 'example.org',
                'tags': 'botnet',
                'lasttime': '2015-01-01T00:00:59Z',
                'comment': 'example comment',
                'attachment': '/tmp/malware.zip'
            }).create()
        """

        self.logger = logging.getLogger(__name__)
        self.client = Client()

        required = {'user', 'feed'}

        if kwargs is None or len(required - set(kwargs.keys())) > 0:
            raise Exception("invalid arguments. missing: {}"
                            .format(required-set(kwargs.keys())))

        self.user = kwargs.pop('user')
        self.feed = kwargs.pop('feed')
        self.comment = kwargs.pop('comment', None)
        self.content = kwargs.pop('content', None)
        self.attachment = kwargs.pop('attachment', None)

        self.indicator = I(**kwargs)

    def show(self, user, feed, id):
        """
        Show a specific indicator by id

        :param user: feed username
        :param feed: feed name
        :param id: indicator endpoint id [INT]
        :return: dict

        Example:
            ret = Indicator.show('csirtgadgets','port-scanners', '1234')
        """
        uri = '/users/{}/feeds/{}/indicators/{}'.format(user, feed, id)
        return self.client.get(uri)

    def _file_to_attachment(self, blob, filename=None):
        """

        :param blob: a local file path or base64 encoded blob
        :param filename: file path
        :return: dict of base64 encoded filestring, with orig filename
        """

        if os.path.isfile(blob):
            filename = blob
            with open(blob, encoding="ISO-8859-1") as f:
                data = f.read()
        else:
            if filename is None:
                raise RuntimeError('missing filename')
            try:
                data = base64.b64decode(blob)
            except TypeError:
                raise RuntimeError('attachment must be base64 encoded')

        md5 = hashlib.md5(data.encode('utf-8')).hexdigest()
        sha1 = hashlib.sha1(data.encode('utf-8')).hexdigest()
        sha256 = hashlib.sha256(data.encode('utf-8')).hexdigest()
        sha512 = hashlib.sha512(data.encode('utf-8')).hexdigest()
        data = base64.b64encode(data.encode('utf-8'))

        return {
            'data': data.decode("utf-8"),
            'filename': filename,
            'sha1': sha1,
            'md5': md5,
            'sha256': sha256,
            'sha512': sha512
        }

    def comments(self, user, feed, id):
        """
        Return comments for a specific indicator id

        :param user: feed username
        :param feed: feed name
        :param id: indicator id [INT]
        :return: list

        Example:
            ret = Indicator.comments('csirtgadgets','port-scanners', '1234')
        """
        uri = '/users/{}/feeds/{}/indicators/{}/comments'\
            .format(user, feed, id)
        return self.client.get(uri)

    def create(self):
        """
        Submit action on the Indicator object

        :return: Indicator Object
        """
        uri = '/users/{0}/feeds/{1}/indicators'\
            .format(self.user, self.feed)

        data = {
            "indicator": json.loads(str(self.indicator)),
            "comment": self.comment,
            "content": self.content
        }

        if self.attachment:
            attachment = self._file_to_attachment(
                self.attachment, filename=self.attachment_name)

            data['attachment'] = {
                'data': attachment['data'],
                'filename': attachment['filename']
            }

        if not data['indicator'].get('indicator'):
            data['indicator']['indicator'] = attachment['sha1']

        if not data['indicator'].get('indicator'):
            raise Exception('Missing indicator')

        return self.client.post(uri, data)

    def create_bulk(self, indicators, user, feed):
        from .constants import API_VERSION
        if API_VERSION == '1':
            print("create_bulk currently un-avail with APIv1")
            raise SystemExit

        """
        Submit action against the IndicatorBulk endpoint

        :param indicators: list of Indicator Objects
        :param user: feed username
        :param feed: feed name
        :return: list of Indicator Objects submitted

        from csirtgsdk.client import Client
        from csirtgsdk.indicator import Indicator

        remote = 'https://csirtg.io/api'
        token = ''
        verify_ssl = True

        i = {
            'indicator': 'example.com',
            'feed': 'test',
            'user': 'admin',
            'comment': 'this is a test',
        }

        data = []

        cli = Client(remote=remote, token=token, verify_ssl=verify_ssl)

        for x in range(0, 5):
            data.append(
                Indicator(cli, i)
            )

        ret = cli.submit_bulk(data, 'csirtgadgets', 'test-feed')
        """

        uri = '/users/{0}/feeds/{1}/indicators_bulk'.format(user, feed)

        data = {
            'indicators': [
                {
                    'indicator': i.args.indicator,
                    'feed_id': i.args.feed,
                    'tag_list': i.args.tags,
                    "description": i.args.description,
                    "portlist": i.args.portlist,
                    "protocol": i.args.protocol,
                    'firsttime': i.args.firsttime,
                    'lasttime': i.args.lasttime,
                    'portlist_src': i.args.portlist_src,
                    'comment': {
                        'content': i.args.comment
                    },
                    'content': i.args.content,
                    'provider': i.args.provider,
                } for i in indicators
                ]
        }
        return self.client.post(uri, data)

    submit = create
    submit_bulk = create_bulk



import pytest

from csirtgsdk.search import Search
from csirtgsdk.client.http import HTTP as Client

@pytest.fixture
def client():
    return Client()


def test_search(client):
    f = Search(client)

    assert f.client


import logging
from csirtgsdk.client.http import HTTP as Client


class Search(object):
    """
    Search Object class
    """
    def __init__(self, client=Client()):
        """

        :param client: client.Client object
        :return: Search Object
        """
        self.logger = logging.getLogger(__name__)
        self.client = client

    def search(self, q, limit=None):
        """
        Performs a search against the /search endpoint

        :param q: query to be searched for [STRING]
        :param limit: limit the results [INT]
        :return: list of dicts
        """
        uri = '{}/search?q={}&limit={}'.format(self.client.remote, q, limit)
        self.logger.debug(uri)

        body = self.client.get(uri)
        return body



import json
import requests
import logging
from csirtgsdk.exceptions import AuthError, TimeoutError, NotFound, \
    SubmissionFailed, RateLimitExceeded, SystemBusy
import os
import random
from time import sleep

from csirtgsdk.constants import API_VERSION, TIMEOUT, REMOTE, TOKEN, \
    VERSION

RETRIES = os.getenv('CSIRTGSDK_CLIENT_HTTP_RETRIES', 5)
RETRIES_DELAY = os.getenv('CSIRTGSDK_CLIENT_HTTP_RETRIES_DELAY', '30,60')

s, e = RETRIES_DELAY.split(',')
RETRIES_DELAY = random.uniform(int(s), int(e))

logger = logging.getLogger(__name__)

if os.getenv('CSIRTGSDK_HTTP_TRACE'):
    logger.setLevel(logging.DEBUG)


class HTTP(object):
    def __init__(self, remote=REMOTE, token=TOKEN, proxy=None,
                 timeout=TIMEOUT, verify_ssl=True):

        self.logger = logger
        self.remote = remote
        self.token = str(token)
        self.proxy = proxy
        self.timeout = timeout
        self.verify_ssl = verify_ssl

        if not self.verify_ssl:
            self.logger.debug('TLS Verification is OFF')

        self.session = requests.session()
        self.session.headers["Accept"] = 'application/vnd.csirtg.v{0}'\
            .format(str(API_VERSION))
        self.session.headers['User-Agent'] = 'csirtgsdk-python/{0}'\
            .format(VERSION)
        self.session.headers['Authorization'] = 'Token token=' + self.token
        self.session.headers['Content-Type'] = 'application/json'
        self.session.headers['Accept-Encoding'] = 'gzip'

    @staticmethod
    def _check_return(resp, expects=[200, 201]):
        if isinstance(expects, int):
            expects = [expects]

        if resp.status_code in expects:
            return True

        if resp.status_code == 401:
            raise AuthError()

        if resp.status_code == 404:
            raise NotFound()

        if resp.status_code == 408:
            raise TimeoutError()

        if resp.status_code == 422:
            raise SubmissionFailed()

        if resp.status_code == 429:
            raise RateLimitExceeded()

        if resp.status_code in [500, 501, 502, 503, 504]:
            raise SystemBusy()

        raise RuntimeError(resp.text)

    def _make_request(self, uri, params={}, data=None):
        n = 0
        while n != RETRIES:
            if len(params) > 0:
                resp = self.session.get(uri, params=params)
            else:
                resp = self.session.post(uri, data=json.dumps(data))

            try:
                self._check_return(resp)
                return json.loads(resp.text)

            except Exception as e:
                logger.error(e)
                if resp.status_code not in [500, 501, 502, 503, 504]:
                    raise e

            logger.info('random retrying in %.00fs' % RETRIES_DELAY)
            sleep(RETRIES_DELAY)

            n += 1

        if n == RETRIES:
            raise SystemBusy("Something went wrong, "
                             "check your data and/or try again later..")

    def _get(self, uri, params={}):
        """
        HTTP GET function

        :param uri: REST endpoint
        :param params: optional HTTP params to pass to the endpoint
        :return: list of results (usually a list of dicts)

        Example:
            ret = cli.get('/search', params={ 'q': 'example.org' })
        """

        if not uri.startswith(self.remote):
            uri = '{}{}'.format(self.remote, uri)

        return self._make_request(uri, params)

    def _post(self, uri, data):
        """
        HTTP POST function

        :param uri: REST endpoint to POST to
        :param data: list of dicts to be passed to the endpoint
        :return: list of dicts, usually will be a list of objects or id's

        Example:
            ret = cli.post('/indicators', { 'indicator': 'example.com' })
        """

        if not uri.startswith(self.remote):
            uri = '{}/{}'.format(self.remote, uri)
            self.logger.debug(uri)

        return self._make_request(uri, data=data)

    post = _post
    get = _get



import pytest
import os

from csirtgsdk.predict import Predict
from time import sleep

CI_BUILD = os.environ.get('CI_BUILD', False)
TOKEN = os.environ.get('CSIRTG_TOKEN', None)
USER = os.environ.get('CSIRTG_USER', 'wes')
REMOTE = os.environ.get('CSIRTG_REMOTE', 'https://csirtg.io/api')

liveonly = pytest.mark.skipif(CI_BUILD is False, reason="CI_BUILD env var not set")


def test_predict():
    f = Predict()

    assert f.client


@liveonly
def test_predict_live():
    sleep(3)
    assert Predict().get("http://example.com") == 0

    assert not Predict().get("http://ren-isac.net")



import pytest
import os
from time import sleep
from csirtgsdk.indicator import Indicator
from csirtgsdk.feed import Feed
from csirtgsdk.client.http import HTTP as Client
from csirtgsdk.search import Search

CI_BUILD = os.environ.get('CI_BUILD', False)
TOKEN = os.environ.get('CSIRTG_TOKEN', None)
USER = os.environ.get('CSIRTG_USER', 'wes')
REMOTE = os.environ.get('CSIRTG_REMOTE', 'https://csirtg.io/api')
FEED = os.environ.get('CSIRTG_TEST_FEED', 'ci_search_test')

liveonly = pytest.mark.skipif(CI_BUILD is False, reason="CI_BUILD env var not set")


@pytest.fixture
def client():
    return Client(
        token=TOKEN,
        remote=REMOTE
    )


@liveonly
def test_indicator_search_fqdn(client):
    sleep(3)
    INDICATOR = 'example123123123.com'

    # create feed and test created feed
    f = Feed(client).new(USER, FEED, description='build search test feed')
    assert f['updated_at']

    # create test and submit test indicator
    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'indicator': INDICATOR,
        'comment': 'this is a test comment'
    })
    r = i.submit()

    # test creating the indicator
    assert r['indicator'] == INDICATOR
    assert r['itype'] == 'fqdn'
    assert r['created_at']

    # search for indicator
    s = Search(client)
    r = s.search(INDICATOR, 10)
    for record in r:
        if record['feed'] == 'live-test-feed':
            assert record['indicator'] == INDICATOR

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200


@liveonly
def test_indicator_search_ipv4(client):
    sleep(3)
    INDICATOR = '1.1.1.1'

    # create feed and test created feed
    f = Feed(client).new(USER, FEED, description='build search test feed')
    assert f['created_at']

    # create test and submit test indicator
    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'indicator': INDICATOR,
        'comment': 'this is a test comment'
    })
    r = i.submit()

    # test creating the indicator
    assert r['indicator'] == INDICATOR
    assert r['itype'] == 'ipv4'
    assert r['created_at']

    # search for indicator
    s = Search(client)
    r = s.search(INDICATOR, 10)
    for record in r:
        if record['feed'] == 'live-test-feed':
            assert record['indicator'] == INDICATOR

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200


@liveonly
def test_indicator_search_ipv6(client):
    sleep(3)
    INDICATOR = '2001:4860:4860::8888'

    # create feed and test created feed
    f = Feed(client).new(USER, FEED, description='build search test feed')
    assert f['created_at']

    # create test and submit test indicator
    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'indicator': INDICATOR,
        'comment': 'this is a test comment'
    })
    r = i.submit()

    # test creating the indicator
    assert r['indicator'] == INDICATOR
    assert r['itype'] == 'ipv6'
    assert r['created_at']

    # search for indicator
    s = Search(client)
    r = s.search(INDICATOR, 10)
    for record in r:
        if record['feed'] == 'live-test-feed':
            assert record['indicator'] == INDICATOR

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200


@liveonly
def test_indicator_search_email(client):
    sleep(3)
    INDICATOR = 'johndoe@example.com'

    # create feed and test created feed
    f = Feed(client).new(USER, FEED, description='build search test feed')
    assert f['created_at']

    # create test and submit test indicator
    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'indicator': INDICATOR,
        'comment': 'this is a test comment'
    })
    r = i.submit()

    # test creating the indicator
    assert r['indicator'] == INDICATOR
    assert r['itype'] == 'email'
    assert r['created_at']

    # search for indicator
    s = Search(client)
    r = s.search(INDICATOR, 10)
    for record in r:
        if record['feed'] == 'live-test-feed':
            assert record['indicator'] == INDICATOR

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200


@liveonly
def test_indicator_search_url(client):
    sleep(3)
    INDICATOR = 'http://www.example.com/test/index.html'

    # create feed and test created feed
    f = Feed(client).new(USER, FEED, description='build search test feed')
    assert f['created_at']

    # create test and submit test indicator
    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'indicator': INDICATOR,
        'comment': 'this is a test comment'
    })
    r = i.submit()

    # test creating the indicator
    assert r['indicator'] == INDICATOR
    assert r['itype'] == 'uri'
    assert r['created_at']

    # search for indicator
    s = Search(client)
    r = s.search(INDICATOR, 10)
    for record in r:
        if record['feed'] == 'live-test-feed':
            assert record['indicator'] == INDICATOR

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200



import pytest
import os
from csirtgsdk.indicator import Indicator
from csirtgsdk.feed import Feed
from csirtgsdk.client.http import HTTP as Client
from time import sleep

CI_BUILD = os.environ.get('CI_BUILD', False)
TOKEN = os.environ.get('CSIRTG_TOKEN', None)
USER = os.environ.get('CSIRTG_USER', 'wes')
REMOTE = os.environ.get('CSIRTG_REMOTE', 'https://csirtg.io/api')
FEED = os.environ.get('CSIRTG_FEED', 'ci_attachment_test')

liveonly = pytest.mark.skipif(CI_BUILD is False, reason="CI_BUILD env var not set")

@pytest.fixture
def client():
    return Client(
        token=TOKEN,
        remote=REMOTE
    )

@liveonly
def test_indicator_attachment_txt(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/message.eml',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    # assert r['indicator'] == '2f9496a6331b2e75e5208b93d144e8fe484b316a'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200

@liveonly
def test_indicator_attachment_zip(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/malware.jar.zip',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    assert r['indicator'] == 'f31e226048d9bd45513e691a50a4b83893397235'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200


@liveonly
def test_indicator_attachment_jar(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/malware.jar',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    assert r['indicator'] == '8ab0079d8e80c2e166b3b12364c89255d79c9f75'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200

@liveonly
def test_indicator_attachment_docx(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/c..docx',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    assert r['indicator'] == 'c2642e519c7f325300ed250710b4f815ac542c1d'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200

@liveonly
def test_indicator_attachment_doc(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/business_relationship.doc',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    assert r['indicator'] == 'f0ee0d5a1279fbdd93a9c5b9a1377894113f0ec0'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200

@liveonly
def test_indicator_attachment_pdf(client):
    sleep(3)
    f = Feed(client).new(USER, FEED, description='test build feed')

    assert f['created_at']

    i = Indicator(client, {
        'user': USER,
        'feed': FEED,
        'attachment': 'samples/hello_world.pdf',
        'comment': 'asdfasdfasdf'
    })
    r = i.submit()

    assert r['indicator'] == 'cc9881dc27a8d3e410cdf7e667ff5efa5cbfdaed'
    # assert r['attachments'][0]['attachment']
    # assert r['attachments'][0]['filesize']
    # assert r['attachments'][0]['created_at']

    # delete test feed
    f = Feed(client).remove(USER, FEED)
    assert f == 200



from csirtgsdk.client.http import HTTP as Client


def test_client():
    c = Client(token=1234, remote='https://localhost2:8443', verify_ssl=False)

    assert c.remote == 'https://localhost2:8443', 'remote incorrect'
    assert c.verify_ssl is False, 'no-verify-ssl incorrect'
    assert c.token == str(1234), 'token mis-match'



from setuptools import setup, find_packages
import os
import sys
import versioneer

with open('requirements.txt') as f:
    reqs = f.read().splitlines()


# https://www.pydanny.com/python-dot-py-tricks.html
if sys.argv[-1] == 'test':
    test_requirements = [
        'pytest',
        'coverage',
        'pytest_cov',
    ]
    try:
        modules = map(__import__, test_requirements)
    except ImportError as e:
        err_msg = e.message.replace("No module named ", "")
        msg = "%s is not installed. Install your test requirments." % err_msg
        raise ImportError(msg)
    r = os.system('py.test test -v --cov=csirtgsdk --cov-fail-under=25')
    if r == 0:
        sys.exit()
    else:
        raise RuntimeError('tests failed')

setup(
    name="csirtgsdk",
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description="CSIRTG Python SDK",
    long_description="CSIRTG Software Development Kit for Python",
    url="https://github.com/csirtgadgets/csirtgsdk-py",
    license='MPL2',
    classifiers=[
        "Topic :: System :: Networking",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        "Programming Language :: Python"
    ],
    keywords=['network','security'],
    author="Wes Young",
    author_email="wes@csirtgadgets.org",
    packages=find_packages(),
    install_requires=[
        'requests',
        'prettytable',
        'pyyaml',
        'tailer',
        'arrow',
        'csirtg_indicator',
        'websocket-client',
    ],
    entry_points={
        'console_scripts': [
            "csirtg=csirtgsdk.client.cli:main",
            'csirtg-firehose=csirtgsdk.client.firehose:main'
        ]
    },
    test_suite="test"
)



import os
import os.path
from argparse import ArgumentParser
from argparse import RawDescriptionHelpFormatter
import textwrap
import logging

from csirtgsdk.feed import Feed
from csirtgsdk.indicator import Indicator
from csirtgsdk.search import Search
from csirtgsdk.predict import Predict
from csirtgsdk.constants import TIMEOUT, REMOTE, LIMIT, TOKEN, COLUMNS
from csirtg_indicator.format import FORMATS

LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s[%(lineno)s] - %(message)s'


def setup_logging(args):
    """
    Sets up basic logging

    :param args: ArgParse arguments
    :return: nothing. sets logger up globally
    """
    loglevel = logging.WARNING
    if args.verbose:
        loglevel = logging.INFO
    if args.debug:
        loglevel = logging.DEBUG

    console = logging.StreamHandler()
    logging.getLogger('').setLevel(loglevel)
    console.setFormatter(logging.Formatter(LOG_FORMAT))
    logging.getLogger('').addHandler(console)


def main():
    parser = ArgumentParser(
        description=textwrap.dedent('''\
        example usage:
            $ csirtg --search example.com
            $ csirtg --user csirtgadgets --feeds
            $ csirtg --user csirtgadgets --feed uce-urls
            $ csirtg --user csirtgadgets --feed-new scanners --description 'a feed of port scanners'
            $ csirtg --user csirtgadgets --feed scanners --indicator-new 1.1.1.1 --tags scanner --comment
              'this is a port scanner'
            $ csirtg --user csirtgadgets --feed uce-attachments --new --attachment 'fax.zip'
              --description 'file attached in uce email'
        '''),
        formatter_class=RawDescriptionHelpFormatter,
        prog='csirtg'
    )

    parser.add_argument("-v", "--verbose", action="count", help="set verbosity level [default: %(default)s]")
    parser.add_argument('-d', '--debug', action="store_true")

    parser.add_argument('--token', help="specify token", default=TOKEN)
    parser.add_argument('-l', '--limit', help="specify results limit [default: %(default)s]", default=LIMIT)
    parser.add_argument('--remote', help="remote api location [default: %(default)s]", default=REMOTE)
    parser.add_argument('--timeout', help='connection timeout [default: %(default)s]', default=TIMEOUT)
    parser.add_argument('-C', '--config', help="configuration file [default: %(default)s]",
                        default=os.path.expanduser("~/.csirtg.yml"))  # env var

    parser.add_argument('--format', help="specify an output format [default: %(default)s]", default='table')
    parser.add_argument('--columns', help='specify output columns [default %(default)s]', default=','.join(COLUMNS))

    # actions
    parser.add_argument('-q', '--search', help="search for an indicator")
    parser.add_argument('--feeds', action="store_true", help="show a list of feeds (per user)")
    parser.add_argument('--new', action='store_true', help="create a new feed or indicator")

    parser.add_argument('--predict', help="test an indicator against the prediction api")
    parser.add_argument('--predict-stdin', action="store_true")

    # vars
    parser.add_argument('--user', help="specify a user")
    parser.add_argument('--feed', help="specify feed name")
    parser.add_argument('--feed-new', help='help specify a new feed')
    parser.add_argument('--indicator', dest='indicator',
                        help="specify an indicator [eg: 1.2.3.4, example.com, "
                        "http://example.com/1.html")
    parser.add_argument('--indicator-new', help='create a new indicator')
    parser.add_argument('--tags', help="specify tags")
    parser.add_argument('--comment', help="enter a comment for the indicator")
    parser.add_argument('--description', help="specify a feed description")

    parser.add_argument('--portlist',
                        help="specify a portlist [eg: 22,23-35,443]")
    parser.add_argument('--protocol', help="specify TCP, UDP or ICMP")
    parser.add_argument('--content')
    parser.add_argument('--provider')

    parser.add_argument('--firsttime',
                        help="timestamp when first seen [eg: 2015-11-23T00:00:00Z]")
    parser.add_argument('--lasttime', help="timestamp when last seen [eg: 2015-11-24T00:00:00Z], treated as 'greater "
                                           "than'")

    parser.add_argument('--attachment', help="specify an attachment [eg: /path/to/file]")
    parser.add_argument('--attachment-name', help="specify the attachment filename")

    parser.add_argument('--no-verify-ssl', help='Turn TLS verification OFF', action="store_true")

    parser.add_argument('--sinkhole', action='store_true')

    # Process arguments
    args = parser.parse_args()
    setup_logging(args)

    logger = logging.getLogger(__name__)

    if args.verbose:
        logger.setLevel(logging.INFO)

    if args.debug:
        logger.setLevel(logging.DEBUG)

    if not TOKEN:
        print('~/.csirtg.yml configuration no longer supported')
        print('Please setup your environment variables:')
        print("")
        print("export CSIRTG_TOKEN=1234..")
        raise SystemExit

    options = vars(args)
    if options.get('sinkhole'):
        import sys
        if not sys.stdin.isatty():
            stdin = sys.stdin.read()
        else:
            logger.error("No data passed via STDIN")
            raise SystemExit

        from csirtgsdk.sinkhole import Sinkhole
        ret = Sinkhole().post(stdin)

        if ret.get('status') == 'unauthorized':
            logger.error('unauthorized')
        else:
            if logger.getEffectiveLevel() == logging.DEBUG:
                print("Predictions: \n")
                for p in ret.get('predictions', []):
                    print("\t%s" % p)

        raise SystemExit

    if options.get('predict') or options.get('predict_stdin'):
        import sys
        if options.get('predict'):
            logger.info("Searching for: {0}".format(options.get('predict')))
            predict = options['predict']
            ret = Predict().get(predict)
            print("Prediction Score: %s - %s" % (ret, predict))
        else:
            import sys
            if not sys.stdin.isatty():
                for predict in sys.stdin.read().split("\n"):
                    if predict == '':
                        continue
                    ret = Predict().get(predict)
                    print("Prediction Score: %s - %s" % (ret, predict))
            else:
                logger.error("No data passed via STDIN")
                raise SystemExit

        raise SystemExit

    if options.get('search'):
        logger.info("Searching for: {0}".format(options.get('search')))
        ret = Search().search(options.get('search'), limit=options['limit'])
        if isinstance(ret, dict):
            ret = [ret]

        print(FORMATS[options.get('format')](data=ret,
                                             cols=args.columns.split(',')))
        raise SystemExit

    if options.get('indicator_new') or options.get('attachment'):
        options['indicator'] = options['indicator_new']
        logger.info("Creating indicator in feed {0} for user {1}"
                    .format(options['feed'], options['user']))
        try:
            ret = Indicator(options).submit()
        except Exception as e:
            print(e)
            raise SystemExit

        logger.info('posted: {0}'.format(ret['location']))

        if isinstance(ret, dict):
            ret = [ret]

        print(FORMATS[options.get('format')](data=ret,
                                             cols=args.columns.split(',')))
        raise SystemExit

    if options.get('feeds'):
        logger.info("Searching feeds for user: {0}".format(options['user']))
        f = Feed()
        feeds = f.index(options['user'])
        for l in f.get_lines(feeds):
            print(l)
        raise SystemExit

    if options.get('feed_new'):
        if not options.get('user'):
            parser.error('--user is required')

        if not options.get('description'):
            parser.error('--description is required')

        logger.info("Creating feed {0} for user {1}"
                    .format(options['feed_new'], options['user']))
        f = Feed()
        feed = f.new(options['user'], options['feed_new'],
                     description=options['description'])
        for l in f.get_lines(feed):
            print(l)

        raise SystemExit

    if options.get('feed'):
        if not options.get('user'):
            parser.error('--user is required')

        logger.info("Fetching feed {0} for user {1}"
                    .format(options['feed'], options['user']))
        data = Feed().show(
                options['user'],
                options['feed'],
                limit=options['limit'],
                lasttime=options['lasttime'],
        )

        if data.get('indicators'):
            print(FORMATS[options.get('format')](data=data['indicators'],
                                                 cols=args.columns.split(',')))


if __name__ == "__main__":
    main()
